----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Jessica Yohana Aguilar Mastranzo
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
-- Description: Las caracteristicas de este modulo son:
--					* 8 switches de control
--					* 8 LEDs 
--					El proyecto consiste en generar un contador binario
--					de 8 bits cuya frecuencia se controla mediante los switches
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.STD_LOGIC_ARITH.ALL; --Librera que contiene la funcin CONV_STD_LOGIC_VECTOR()

entity aplicacion_conta_switch is

PORT (	CLK    		: IN  STD_LOGIC;                    --Reloj FPGA a 100MHz.
			LEDS   		: OUT STD_LOGIC_VECTOR(7 DOWNTO 0); --LEDs de la tarjeta donde se observar 
																			--el conteo.
			SWITCH 		: IN  STD_LOGIC_VECTOR(7 DOWNTO 0)  --Switches de la tarjeta para cambiar la 
																			--frecuencia del conteo
	  );
		
end aplicacion_conta_switch;

architecture Behavioral of aplicacion_conta_switch is

--- SEALES PARA GENERAR RETARDO ---
signal bandera       : std_logic := '0'; --Bandera que indica cuando ya se gener el retardo.
signal escala        : integer range 0 to 100_000_000 := 0; --Valor de escala que cambia dependiendo 
																				--el valor de switches. El contador del 
																				--retardo llegar a este valor.
signal conta_retardo : integer range 0 to 100_000_000 := 0; --Contador para generar retardo
---
signal conta_leds 	: integer range 0 to 255 := 0;	--Contador principal y que se mandar a los LEDs. 
																		--Se convertir a un vector.

begin
-- proceso que genera retardo --
process(CLK)
begin
	if rising_edge(CLK) then
		conta_retardo <= conta_retardo+1;
		if conta_retardo >= escala-1 then
			bandera <= '1';
			conta_retardo <= 0;
		else
			bandera <= '0';
		end if;
	end if;
end process;
---

-- Proceso que genera el conteo principal --
process(CLK)
begin
	if rising_edge(CLK) then
		if bandera = '1' then
			conta_leds <= conta_leds+1;
			if conta_leds = 255 then
				conta_leds <= 0;
			end if;
		end if;
	end if;
end process;
	
--Sentencia concurrente que le asigna a "escala" diferentes valores --
with SWITCH select
  escala <= 80_000_000  when "00000001", 
            60_000_000  when "00000011",
            40_000_000  when "00000111", 
            30_000_000  when "00001111", 
            20_000_000  when "00011111", 
            10_000_000  when "00111111", 
            2_000_000   when "01111111",			 
            100_000_000 when others;
        
LEDS <= conv_std_logic_vector(conta_leds,8); --Convierte el contador "integer" en "std_logic_vector".        

end behavioral;